@@ -1,21 +1,22 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/display.c,v 1.3 2002/06/14 21:02:41 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/display.c,v 1.19 2003/03/25 19:26:10 richb Exp $
  *
- *  Copyright (c) 1987-2002, Sun Microsystems, Inc.  All Rights Reserved.
- *  Sun considers its source code as an unpublished, proprietary
- *  trade secret, and it is available only under strict license
- *  provisions.  This copyright notice is placed here only to protect
- *  Sun in the event the source is deemed a published work.  Dissassembly,
- *  decompilation, or other means of reducing the object code to human
- *  readable form is prohibited by the license agreement under which
- *  this code is provided to the user or company in possession of this
- *  copy.
- *
- *  RESTRICTED RIGHTS LEGEND: Use, duplication, or disclosure by the
- *  Government is subject to restrictions as set forth in subparagraph
- *  (c)(1)(ii) of the Rights in Technical Data and Computer Software
- *  clause at DFARS 52.227-7013 and in similar clauses in the FAR and
- *  NASA FAR Supplement.
+ *  Copyright (c) 1987-2003 Sun Microsystems, Inc. All Rights Reserved.
+ *           
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2, or (at your option)
+ *  any later version.
+ *           
+ *  This program is distributed in the hope that it will be useful, but 
+ *  WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
+ *  General Public License for more details.
+ *           
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ *  02111-1307, USA.
  */
 
 #include <stdio.h>
@@ -43,7 +44,7 @@ char_val(char chr)
 
 
 void
-clear_display()
+clear_display(int initialise)
 {
     int i;
 
@@ -51,51 +52,37 @@ clear_display()
     v->toclear = 1;
     i = 0;
     mpcim(&i, v->MPdisp_val);
-    STRCPY(v->display, make_number(v->MPdisp_val));
-    set_item(DISPLAYITEM, v->display);
+    STRCPY(v->display, make_number(v->MPdisp_val, FALSE));
+    set_display(v->display);
 
-    v->hyperbolic = 0;
-    v->inverse    = 0;
-    v->show_paren = 0;
-    v->opsptr     = 0;            /* Clear parentheses stacks. */
-    v->numsptr    = 0;
-    set_item(HYPITEM, "    ");
-    set_item(INVITEM, "    ");
+    if (initialise == TRUE) {
+        v->show_paren = 0;
+        v->opsptr     = 0;            /* Clear parentheses stacks. */
+        v->numsptr    = 0;
+        set_hyp_item(FALSE);          /* Also clears v->hyperbolic. */
+        set_inv_item(FALSE);          /* Also clears v->inverse. */
+    }
 }
 
 
-void
-get_label(int n, int *spaces, int *nchars)
+struct button *
+copy_button_info(struct button *old)
 {
-    int i, len, mod, val;
+    struct button *new;
 
-    mod = button_mods(n);
-    val = button_value(n);
-    if (v->tstate) {
-        if (mod == GDK_CONTROL_MASK) {
-            SPRINTF(v->pstr, "^%c  ", val);
-        } else if (val == GDK_Delete) {
-            STRCPY(v->pstr, _("del"));
-        } else if (val == GDK_BackSpace) {
-            STRCPY(v->pstr, _("bsp"));
-        } else if (val == GDK_caret) {
-            STRCPY(v->pstr, _("^"));
-        } else {
-            SPRINTF(v->pstr, "%c   ", val);
-        }
-    } else {
-        STRCPY(v->pstr, button_str(n));
-    }
-    *spaces = 0;
-    len = strlen(v->pstr);
-    for (i = len-1; i >= 0; i--) {
-        if (v->pstr[i] == ' ') {
-            (*spaces)++;
-        } else {
-            break;
-        }
-    }
-    *nchars = len - *spaces;
+    new = malloc(sizeof(struct button));
+
+/* Note that copies of the strings aren't done here as they aren't needed. */
+
+    new->str = old->str;
+    new->hstr = old->hstr;
+    new->mods = old->mods;
+    new->value = old->value;
+    new->func_char = old->func_char;
+    new->mtype = old->mtype;
+    new->func = old->func;
+
+    return(new);
 }
 
 
@@ -127,7 +114,9 @@ make_fixed(int *MPnumber,     /* Convert MP number to fixed number string. */
     mpabs(MPnumber, MPval);
     n = 0;
     mpcim(&n, MP1);
-    if (mplt(MPnumber, MP1)) *optr++ = '-';
+    if (mplt(MPnumber, MP1)) {
+        *optr++ = '-';
+    }
 
     mpcim(&basevals[(int) v->base], MP1base);
 
@@ -156,6 +145,11 @@ make_fixed(int *MPnumber,     /* Convert MP number to fixed number string. */
         }
         mpmul(MPval, MP1base, MPval);
         mpcmi(MPval, &dval);
+
+        if (dval > basevals[(int) v->base]-1) {
+            dval = basevals[(int) v->base]-1;
+        }
+
         *optr++ = digits[dval];
         dval = -dval;
         mpaddi(MPval, &dval, MPval);
@@ -164,12 +158,25 @@ make_fixed(int *MPnumber,     /* Convert MP number to fixed number string. */
     v->toclear = 1;
     v->pointed = 0;
 
+    if (!v->show_zeroes && v->accuracy != 0) {
+        optr = v->fnum + strlen(v->fnum) - 1;
+        while (*optr == '0') {
+            optr--;
+        }
+        if (*optr != '.') {
+            optr++;
+        }
+        *optr = '\0';
+    }
+
     return(v->fnum);
 }
 
 
+/* Convert MP number to character string. */
+
 char *
-make_number(int *MPnumber)     /* Convert MP number to character string. */
+make_number(int *MPnumber, BOOLEAN mkFix)
 {
     double number, val;
 
@@ -190,6 +197,12 @@ make_number(int *MPnumber)     /* Convert MP number to character string. */
         (v->dtype == SCI) ||
         (v->dtype == FIX && val != 0.0 && (val > max_fix[(int) v->base]))) {
         return(make_eng_sci(MPnumber));
+    } else if (v->dtype == FIX && val != 0.0 && mkFix) {
+        if (val <= min_fix[v->accuracy][(int) v->base]) {
+            return(make_eng_sci(MPnumber));
+        } else {
+            return(make_fixed(MPnumber, MAX_DIGITS));
+        }
     } else {
         return(make_fixed(MPnumber, MAX_DIGITS));
     }
@@ -366,21 +379,18 @@ paren_disp(char c)
  *  character was a left parenthesis, decrement the parentheses count. If
  *  the parentheses count is zero, exit parenthesis processing.
  *
- *  If the character is a control character (not Ctrl-h), then append ^(char).
- *
  *  Otherwise just append the character.
  */
 
     n = strlen(v->display);
-    if (IS_KEY(c, KEY_CLR)) {             /* Is it a Delete character? */
+    if (c == -1) {         /* Is it a Delete character? */
         v->noparens = v->pending = v->opsptr = v->numsptr = 0;
         v->cur_op = '?';
-        set_item(OPITEM, "");
         i = 0;
         mpcim(&i, v->MPdisp_val);
         show_display(v->MPdisp_val);
         return;
-    } else if (IS_KEY(c, KEY_BSP)) {      /* Is is a Back Space character? */
+    } else if (c == 8) {  /* Is is a Back Space character? */
         if (!n) {
             return;
         }
@@ -389,18 +399,11 @@ paren_disp(char c)
             if (!v->noparens) {
                 v->pending = v->opsptr = v->numsptr = 0;
                 v->cur_op = '?';
-                set_item(OPITEM, "");
                 show_display(v->MPdisp_val);
                 return;
             }
         }
         v->display[n-1] = '\0';
-    } else if (c <= CTL('z')) {           /* Is it a control character? */
-        if (n < MAXLINE-2) {
-            v->display[n]   = '^';
-            v->display[n+1] = c + 96;
-            v->display[n+2] = '\0';
-        }
     } else {                           /* It must be an ordinary character. */
         if (n < MAXLINE-1) {
             v->display[n]   = c;
@@ -409,39 +412,38 @@ paren_disp(char c)
     }
 
     n = (n < MAX_DIGITS) ? 0 : n - MAX_DIGITS;
-    v->show_paren = 1;       /* Hack to get set_item to really display it. */
-    set_item(DISPLAYITEM, &v->display[n]);
+    v->show_paren = 1;       /* Hack to get set_display to really display it. */
+    set_display(&v->display[n]);
     v->show_paren = 0;
 }
 
 
 void
-process_item(int n)
+process_item(struct button *button)
 {
-    int i,isvalid;
+    int i, isvalid;
 
-    if (n < 0 || n >= NOBUTTONS) {
-        if (v->beep == TRUE) {
-            beep();
-        }
-        return;
+    if (v->current != NULL) {
+        free(v->current);
     }
+    v->current = copy_button_info(button);
 
-    v->current = button_value(n);
-    if (v->current == '*') {
-        v->current = 'x';      /* Reassign "extra" values. */
+/* Reassign "extra" values. */
+
+    if (v->current->value == 'x') {
+        v->current->value = '*';
     }
-    if (v->current == GDK_Return) {
-        v->current = '=';
+    if (v->current->value == GDK_Return) {
+        v->current->value = '=';
     }
-    if (v->current == 'Q') {
-        v->current = 'q';
+    if (v->current->value == 'Q') {
+        v->current->value = 'q';
     }
 
     if (v->error) {
         isvalid = 0;                    /* Must press a valid key first. */
         for (i = 0; i < MAXVKEYS; i++) {
-            if (v->current == validkeys[i]) {
+            if (v->current->value == validkeys[i]) {
                 isvalid = 1;
             }
         }
@@ -455,35 +457,13 @@ process_item(int n)
     }
 
     if (v->pending) {
-        if (v->pending_win == FCP_KEY) {
-            (*buttons[v->pending_n].func)();
-        } else {
-            (*mode_buttons[MODEKEYS * ((int) v->pending_mode - 1) +
-            v->pending_n].func)();
-        }
-        return; 
+        (*v->pending_but->func)();
+        return;
     }
-    STRCPY(v->opstr, v->item_text[(int) OPITEM]);
-    switch (button_opdisp(n)) {
-        case OP_SET : 
-            set_item(OPITEM, button_str(n));
-            break;
 
-        case OP_CLEAR : 
-            if (v->error) {
-                set_item(OPITEM, _("CLR"));
-            } else {
-                set_item(OPITEM, "");
-            }
+    STRCPY(v->opstr, v->op_item_text);
 
-        default : 
-            break;
-    }
-    if (v->curwin == FCP_KEY) {
-        (*buttons[n].func)();
-    } else {
-        (*mode_buttons[MODEKEYS * ((int) v->modetype - 1) + n].func)();
-    }
+    (*button->func)();
 }
 
 
@@ -491,26 +471,7 @@ void
 show_display(int *MPval)
 {
     if (!v->error) {
-        STRCPY(v->display, make_number(MPval));
-        set_item(DISPLAYITEM, v->display);
-    }
-}
-
-
-void
-switch_hands(int righthand)
-{
-    int i, j, n;
-
-    for (i = 0; i < BROWS; i++) {
-        for (j = 0; j < BCOLS; j++) {
-            n = (righthand) ? right_pos[j] : left_pos[j];
-            MEMCPY((char *) &v->temp_buttons[n],
-                   (char *) &buttons[i*BCOLS + j], sizeof(struct button));
-        }
-        for (j = 0; j < BCOLS; j++) {
-            MEMCPY((char *) &buttons[i*BCOLS + j], 
-                   (char *) &v->temp_buttons[j], sizeof(struct button));
-        }
+        STRCPY(v->display, make_number(MPval, TRUE));
+        set_display(v->display);
     }
 }
